## Introduction

The document talks about the plan to move from Event Grid to Service Bus. The major consumer of the same iss
Notification Service. As Notification Service is consumed by external customers, so a clean migration path is needed.

## Goals

The Migration must happen respecting the following

1. No re-registration
2. Zero downtime.
3. No notification loss.

#### Prerequisites

1. Please verify that the topic you want to use exists. If not,
   follow [this](https://community.opengroup.org/osdu/platform/system/notification/-/blob/master/provider/notification-azure/PLAYBOOK_FOR_TOPIC_CREATION.md)
   guide to create one.
2. Install
   the [latest version](https://docs.microsoft.com/en-us/powershell/scripting/install/installing-powershell?view=powershell-7.1)
   of PowerShell available for your operating system.

## How to do the migration

Migration is something which must be performed by SRE. The flags must be switched in a sequence.The flags are controlled
by deployment.yaml for the service. Following are the steps needed for successful migration.

1. Have 2 deployment of Notification Service running before migration:
    1. Deployment 1,having event_grid_enabled.
    2. Deployment 2,having service_bus_enabled.The consecutive_subscription_manager_delay_seconds should not be set much
       higher as it will pile up alot of SB messages because of the longer time of listening new subscribers on the fly.
       **The number of pods for both the deployments should be managed as per the load during migration**
2. Get list of subscriptions per partition and prepare migrationConfig.json [Manual-from azure portal]:
3. Complete powershell Az module installation and Azure login by running script
   - [migrationSetup.ps1](https://community.opengroup.org/osdu/platform/system/notification/-/blob/master/provider/notification-azure/src/main/resources/migrationSetup.ps1)
   .

**Note:The Azure Az PowerShell module works with PowerShell 7.x and later on all platforms.To check your PowerShell
version, run the following command from within a PowerShell session:$PSVersionTable.PSVersion**

4. Create the subscribers on Service Bus with the same notification id as in CosmosDB by providing migrationConfig.json
   to the
   script-[migrationToServiceBus.ps1](https://community.opengroup.org/osdu/platform/system/notification/-/blob/master/provider/notification-azure/src/main/resources/migrationToServiceBus.ps1)
   . All the subscriptions from one Event grid topic will be moved to service bus topic for one execution of above
   mentioned script.We need to rerun script for multiple topic across multiple partitions.
5. Turn on **Service bus flags** and turn off **Event Grid flags** in Producer Services. For example Storage service
   publishes to Service bus and Event grid both as of now. During migration,while we are moving to Service bus
   completely we have to stop publishing to Event Grid as a part of migration, so we can eventually disable it and move
   to Service Bus completely.Same applies to other producer services like -Legal, Schema
   etc. [Manual- Configuring deployment.yaml for Legal, Schema, Storage Services]
6. Wait for Event Grid subscribers to drain.[Manual-from azure portal]
   **Note: If there are delivery failures,then retry can occur after a longer duration,EG should be kept enabled for
   sufficiently long time**
7. Turn on **Service bus flags** in Register Service.[Manual]
8. Event Grid Subscriber clean up.[Manual-from azure portal]

The 3rd and 4th steps would be performed by the scripts in the order given below :
** Powershell must be elevated to admin.**

1. [migrationSetup.ps1](https://community.opengroup.org/osdu/platform/system/notification/-/blob/master/provider/notification-azure/src/main/resources/migrationSetup.ps1)
2. [migrationToServiceBus.ps1](https://community.opengroup.org/osdu/platform/system/notification/-/blob/master/provider/notification-azure/src/main/resources/migrationToServiceBus.ps1)

## List of Feature Flags to be used from deployment.yaml of respective services

1. Notification Service
    1. service_bus_enabled
    2. event_grid_enabled
2. Register Service
    1. azure_serviceBus_enabled
    2. azure_eventGrid_enabled
3. Producer Services
    1. Storage Service : azure_publishToEventGrid
    2. Legal Service : azure.publishToEventGrid
    3. Schema Service : event_grid_enabled

## Post Migration Plan

1. Redeploy Notification Service to have only 1 deployment with service_bus_enabled set to true and event_grid_enabled
   set to false.The number of pods can be managed as per the load

## Note

There would be duplication of notifications that accounts for the time when producer services will be publishing to both
Service bus and Event grid at any time during migration.For example Storage service by default publish to both.We have
to use Publish flags carefully to avoid duplication  