// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure.util;

import static org.opengroup.osdu.core.common.http.ResponseHeaders.STANDARD_RESPONSE_HEADERS;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import java.util.Map;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.opengroup.osdu.notification.provider.interfaces.IPubsubHandshakeHandler;
import org.opengroup.osdu.notification.provider.interfaces.IPubsubRequestBodyExtractor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;

/**
 * TODO: logic needs to be reviewed to avoid looping dependency happening in EventGridRequestBodyExtractor for RQs
 * handshake if any other filters try to inject DpsHeaders before this filter, up for now  set to lowest value for
 * order
 */
@Component
@Order(-104)
public class HandshakeFilter implements Filter {

  @Autowired
  private IPubsubRequestBodyExtractor requestBodyExtractor;

  @Autowired
  private IPubsubHandshakeHandler handshakeHandler;

  @Override
  public void init(FilterConfig filterConfig) {
    //do nothing
  }

  @Override
  public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
      throws IOException, ServletException {
    HttpServletRequest request = (HttpServletRequest) servletRequest;
    if (request.getServletPath().contains("records-changed") && this.requestBodyExtractor.isHandshakeRequest()) {
      HttpServletResponse httpServletResponse = (HttpServletResponse) servletResponse;
      httpServletResponse.setStatus(200);
      httpServletResponse.setContentType(MediaType.APPLICATION_JSON_VALUE);
      setResponseHeaders(httpServletResponse);

      String handshakeResponse = this.handshakeHandler.getHandshakeResponse();
      PrintWriter out = httpServletResponse.getWriter();
      out.flush();
      out.print(handshakeResponse);
      return;
    }
    filterChain.doFilter(servletRequest, servletResponse);
  }

  @Override
  public void destroy() {
  }

  private void setResponseHeaders(HttpServletResponse httpServletResponse) {
    for (Map.Entry<String, List<Object>> header : STANDARD_RESPONSE_HEADERS.entrySet()) {
      httpServletResponse.addHeader(header.getKey(), header.getValue().toString());
    }
  }
}
