/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.provider.gcp.pubsub;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.gcp.oqm.model.OqmAckReplier;
import org.opengroup.osdu.core.gcp.oqm.model.OqmMessage;
import org.opengroup.osdu.core.gcp.oqm.model.OqmSubscription;
import org.opengroup.osdu.core.gcp.oqm.model.OqmTopic;
import org.opengroup.osdu.notification.provider.gcp.pubsub.receiver.OqmPublishTopicReceiver;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class OqmPublishTopicReceiverTest {

    @InjectMocks
    private OqmPublishTopicReceiver sut;
    @Mock
    private OqmAckReplier replier;
    @Mock
    private OqmNotificationHandler notificationHandler;
    @Mock
    private HttpResponse httpResponse;
    @Mock
    private OqmSubscription subscription;
    private final OqmTopic topic = OqmTopic.builder().name("topic1").build();
    private final List<OqmTopic> topics = Collections.singletonList(topic);


    @Test
    public void testReceiveMessageNotificationSuccess() throws Exception {
        when(notificationHandler.notifySubscriber(any(), any(), anyMap())).thenReturn(httpResponse);
        when(httpResponse.isSuccessCode()).thenReturn(true);
        when(subscription.getTopics()).thenReturn(topics);
        OqmMessage createMessage = OqmMessage.builder().data("Subscription Created").attributes(new HashMap<>()).build();
        sut.receiveMessage(createMessage, replier);
        verify(replier, times(1)).ack();
    }

    @Test
    public void testReceiveMessageNotificationFailed() throws Exception {
        when(notificationHandler.notifySubscriber(any(), any(), anyMap())).thenReturn(httpResponse);
        when(httpResponse.isSuccessCode()).thenReturn(false);
        when(subscription.getTopics()).thenReturn(topics);
        OqmMessage createMessage = OqmMessage.builder().data("Subscription Created").attributes(new HashMap<>()).build();
        sut.receiveMessage(createMessage, replier);
        verify(replier, times(1)).nack();
    }

    @Test
    public void testReceiveMessageException() throws Exception {
        when(notificationHandler.notifySubscriber(any(), any(), anyMap())).thenThrow(new Exception());
        when(subscription.getTopics()).thenReturn(topics);
        OqmMessage unsupportedMessage = OqmMessage.builder().data("Unsupported").attributes(new HashMap<>()).build();
        sut.receiveMessage(unsupportedMessage, replier);
        verify(replier, times(1)).nack();
    }
}